package org.test;

import com.sun.star.frame.XDesktop;
import com.sun.star.frame.XModel;
import com.sun.star.lang.IllegalArgumentException;
import com.sun.star.lang.XMultiComponentFactory;
import com.sun.star.reflection.InvocationTargetException;
import com.sun.star.script.provider.ScriptFrameworkErrorException;
import com.sun.star.script.provider.XScript;
import com.sun.star.script.provider.XScriptProvider;
import com.sun.star.script.provider.XScriptProviderFactory;
import com.sun.star.uno.Exception;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;

public class ExecuteBasicScript {

	private XComponentContext xComponentContext = null;

	public ExecuteBasicScript(final XComponentContext xComponentContext) {
		this.xComponentContext = xComponentContext;
	}

	public Object executeScript(final String script, final Object[] aArgs) {

		try {
			final Object oMasterScriptProviderFactory = ExecuteBasicScript
					.createInstanceWithContext(this.xComponentContext,
							"com.sun.star.script.provider.MasterScriptProviderFactory");

			final XScriptProviderFactory xScriptProviderFactory = ExecuteBasicScript
					.toScriptProviderFactory(oMasterScriptProviderFactory);

			if (xScriptProviderFactory == null)
				return null;

			final XScriptProvider xScriptProvider = xScriptProviderFactory
					.createScriptProvider(getModel());

			if (xScriptProvider == null)
				return null;

			final XScript xScript = xScriptProvider.getScript(script);

			if (xScript == null)
				return null;

			final Object ret = xScript.invoke(aArgs, null, null);

			return ret;

		} catch (final IllegalArgumentException e) {
			System.err.println(e.getMessage());
		} catch (final ScriptFrameworkErrorException e) {
			System.err.println(e.getMessage());
		} catch (final InvocationTargetException e) {
			System.err.println(e.getMessage());
		} catch (final NullPointerException e) {
			System.err.println(e.getMessage());
		} catch (final Exception e) {
			System.err.println(e.getMessage());
		}
		return null;
	}

	public static Object createInstanceWithContext(
			final XComponentContext xComponentContext, final String service)
			throws Exception {

		if (xComponentContext != null) {
			final XMultiComponentFactory xMultiComponentFactory = xComponentContext
					.getServiceManager();
			if (xMultiComponentFactory != null)
				return xMultiComponentFactory.createInstanceWithContext(
						service, xComponentContext);
		}
		return null;
	}

	public static XScriptProviderFactory toScriptProviderFactory(
			final Object obj) {
		if (obj != null)
			return UnoRuntime.queryInterface(XScriptProviderFactory.class, obj);

		return null;
	}

	public XModel getModel() {

		final XDesktop xDesktop = getDesktop();

		if (xDesktop != null)
			return toModel(xDesktop.getCurrentComponent());
		return null;
	}

	public XModel toModel(final Object obj) {
		if (obj != null)
			return UnoRuntime.queryInterface(XModel.class, obj);
		return null;
	}

	public static XDesktop toDesktop(final Object obj) {
		if (obj != null)
			return UnoRuntime.queryInterface(XDesktop.class, obj);
		return null;
	}

	public XDesktop getDesktop() {
		try {
			return ExecuteBasicScript.toDesktop(ExecuteBasicScript
					.createInstanceWithContext(this.xComponentContext,
							"com.sun.star.frame.Desktop"));
		} catch (final Exception e) {
			return null;
		}
	}
}
