package org.test;

import com.sun.star.awt.Rectangle;
import com.sun.star.awt.WindowAttribute;
import com.sun.star.awt.WindowClass;
import com.sun.star.awt.WindowDescriptor;
import com.sun.star.awt.XMessageBox;
import com.sun.star.awt.XToolkit;
import com.sun.star.awt.XWindowPeer;
import com.sun.star.frame.DispatchDescriptor;
import com.sun.star.frame.XDispatch;
import com.sun.star.frame.XDispatchProvider;
import com.sun.star.frame.XFrame;
import com.sun.star.frame.XStatusListener;
import com.sun.star.lang.XInitialization;
import com.sun.star.lang.XServiceInfo;
import com.sun.star.lib.uno.helper.WeakBase;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;

public class OOProtocolHandler extends WeakBase implements XDispatchProvider,
		XDispatch, XInitialization, XServiceInfo {

	static private final String[] m_serviceNames = { "com.sun.star.frame.ProtocolHandler" };

	private final XComponentContext m_xCmpCtx;
	private XToolkit m_xToolkit;
	private XFrame m_xFrame;

	public OOProtocolHandler(final XComponentContext xComponentContext) {
		this.m_xCmpCtx = xComponentContext;
	}

	public void initialize(final Object[] object)
			throws com.sun.star.uno.Exception {

		if (object.length > 0)
			this.m_xFrame = UnoRuntime.queryInterface(XFrame.class, object[0]);

		// Create the toolkit to have access to it later
		this.m_xToolkit = UnoRuntime.queryInterface(XToolkit.class,
				this.m_xCmpCtx.getServiceManager().createInstanceWithContext(
						"com.sun.star.awt.Toolkit", this.m_xCmpCtx));
	}

	public String[] getSupportedServiceNames() {
		return OOProtocolHandler.getServiceNames();
	}

	public static String[] getServiceNames() {
		return OOProtocolHandler.m_serviceNames;
	}

	public boolean supportsService(final String sService) {
		final int len = OOProtocolHandler.m_serviceNames.length;

		for (int i = 0; i < len; i++)
			if (sService.equals(OOProtocolHandler.m_serviceNames[i]))
				return true;
		return false;
	}

	public String getImplementationName() {
		return OOProtocolHandler.class.getName();
	}

	// XDispatchProvider
	public XDispatch queryDispatch( /* IN */final com.sun.star.util.URL aURL,
	/* IN */final String sTargetFrameName,
	/* IN */final int iSearchFlags) {
		XDispatch xRet = null;
		if (aURL.Protocol.compareTo("test.java.oo:") == 0) {
			if (aURL.Path.compareTo("Function1") == 0)
				xRet = this;
			if (aURL.Path.compareTo("Function2") == 0)
				xRet = this;
		}
		return xRet;
	}

	public XDispatch[] queryDispatches(
	/* IN */final DispatchDescriptor[] seqDescripts) {
		final int nCount = seqDescripts.length;
		final XDispatch[] lDispatcher = new XDispatch[nCount];

		for (int i = 0; i < nCount; ++i)
			lDispatcher[i] = queryDispatch(seqDescripts[i].FeatureURL,
					seqDescripts[i].FrameName, seqDescripts[i].SearchFlags);

		return lDispatcher;
	}

	// XDispatch
	public void dispatch( /* IN */final com.sun.star.util.URL aURL,
	/* IN */final com.sun.star.beans.PropertyValue[] aArguments) {

		if (aURL.Protocol.compareTo("test.java.oo:") == 0) {
			if (aURL.Path.compareTo("Function1") == 0)
				new ExecuteBasicScript(this.m_xCmpCtx)
						.executeScript(
								"vnd.sun.star.script:Standard.Module1.Test?language=Basic&location=application",
								new Object[] {});

			if (aURL.Path.compareTo("Function2") == 0)
				showMessageBox("SDK DevGuide Add-On example",
						"Java Function activated");
		}
	}

	public void addStatusListener( /* IN */final XStatusListener xControl,
	/* IN */final com.sun.star.util.URL aURL) {
	}

	public void removeStatusListener( /* IN */final XStatusListener xControl,
	/* IN */final com.sun.star.util.URL aURL) {
	}

	public void showMessageBox(final String sTitle, final String sMessage) {
		try {
			if (null != this.m_xFrame && null != this.m_xToolkit) {

				// describe window properties.
				final WindowDescriptor aDescriptor = new WindowDescriptor();
				aDescriptor.Type = WindowClass.MODALTOP;
				aDescriptor.WindowServiceName = new String("infobox");
				aDescriptor.ParentIndex = -1;
				aDescriptor.Parent = UnoRuntime.queryInterface(
						XWindowPeer.class, this.m_xFrame.getContainerWindow());
				aDescriptor.Bounds = new Rectangle(0, 0, 300, 200);
				aDescriptor.WindowAttributes = WindowAttribute.BORDER
						| WindowAttribute.MOVEABLE | WindowAttribute.CLOSEABLE;

				final XWindowPeer xPeer = this.m_xToolkit
						.createWindow(aDescriptor);
				if (null != xPeer) {
					final XMessageBox xMsgBox = UnoRuntime.queryInterface(
							XMessageBox.class, xPeer);
					if (null != xMsgBox) {
						xMsgBox.setCaptionText(sTitle);
						xMsgBox.setMessageText(sMessage);
						xMsgBox.execute();
					}
				}
			}
		} catch (final com.sun.star.uno.Exception e) {
			// do your error handling
		}
	}
}